
NODEFAULT: str = "REQUIRED: NO_DEFAULT"
DEFAULT_SYNC_CURRENT_FIRST: bool = True
DEFAULT_BILL_MANAGMENT: bool = False

DEFAULT_CLI_HOST: str = '127.0.0.1'
DEFAULT_CLI_PORT: int = 8990
DEFAULT_CLI_TIMEOUT: int = 30
DEFAULT_CLI_AUTH: bool = True
DEFAULT_CLI_USERNAME: str = "jcliadmin"
DEFAULT_CLI_PASSWORD: str = "jclipwd"
DEFAULT_CLI_STANDARD_PROMPT: str = "jcli : "
DEFAULT_CLI_INTERACTIVE_PROMPT: str = "> "

DEFAULT_LOG_PATH: str = "/var/log/jasmin/"
DEFAULT_LOG_LEVEL: str = "INFO"
